//@author: vvvv group
//@help: This is a very basic template. Use it to start writing your own effects. If you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects
//@tags: hlsl
//@credits:

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

static const float2 filters = float2(.5, .9);
float epsilon;
int _LinearColorSpace;
float4 TexelSize = float4(0.0019,0.0023,512,424) ;
int MeshDensity = 128;
float4 Crop   = float4 (0.22,0.321,0.5968, 0.3686);
float FarClip ; 
float NearClip ; 
float2 ImageDimensions ; 
float2 PrincipalPoint ; 
float2 depthFocalLength;
float4x4 extrinsics;
float multer;
float2 adder;
//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float3 rgb2hsv(float3 c)
{
	float4 K = float4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	float4 p = lerp(float4(c.bg, K.wz), float4(c.gb, K.xy), step(c.b, c.g));
	float4 q = lerp(float4(p.xyw, c.r), float4(c.r, p.yzx), step(p.x, c.r));

	float d = q.x - min(q.w, q.y);
	return float3(abs(q.z + (q.w - q.y) / (6.0 * d + epsilon)), d / (q.x + epsilon), q.x);
}

float depthForPoint(float2 texturePoint){

	float4 textureSample = float4( texturePoint.x, texturePoint.y, 0.0, 0.0);
	float4 depthsample = pow( tex2Dlod(Samp, textureSample), _LinearColorSpace == 1 ? 0.4545 : 1.0);
	float3 depthsamplehsv = rgb2hsv(depthsample.rgb);

	return depthsamplehsv.g > filters.x && depthsamplehsv.b > filters.y ? depthsamplehsv.r : 0.0;
}

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 ColTexCd : TEXCOORD0;
	float2 DepTexCd : TEXCOORD1;
	float valid  : TEXCOORD3;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(vs2ps In)
 //   float4 PosO  : POSITION,
 //   float2 ColTexCd : TEXCOORD0,
//	float2 DepTexCd : TEXCOORD1,
//	float valid : TEXCOORD2)

{
    //declare output struct
    vs2ps Out;
	
	float4 vertIn = In.Pos;
	float2 depthTexCoord = In.DepTexCd;
	float2 colorTexCoord = In.ColTexCd;
	
	float2 centerpix = TexelSize.xy * 0.5 + adder  ;
	float2 textureStep = float2( 1.0 / (MeshDensity - 1.0), 1.0 / (MeshDensity - 1.0) );
    float2 basetex = floor(vertIn.xy * TexelSize.zw) * TexelSize.xy;
	
	depthTexCoord = multer* basetex * float2(1.0, 0.5) + centerpix;
	colorTexCoord =multer* basetex * float2(1.0, 0.5) + float2(0.0, 0.5) + centerpix;

		//check neighbors
		float2 neighbors[8] = {
		float2(			   0.,  textureStep.y),
		float2( textureStep.x,	 		   0.),
		float2(			   0., -textureStep.y),
		float2(-textureStep.x,			   0.),
		float2(-textureStep.x, -textureStep.y),
		float2( textureStep.x,  textureStep.y),
		float2( textureStep.x, -textureStep.y),
		float2(-textureStep.x,  textureStep.y)
	};
	
 	//texture coords come in as [0.0 - 1.0] for this whole plane
	float depth = depthForPoint(depthTexCoord);
    
	int i;
	float neighborDepths[8];
	for (i = 0; i < 8; i++) {
		neighborDepths[i] = depthForPoint(depthTexCoord + neighbors[i]);
	}

	//den exw idea ti einai auto
	float valium = In.valid;
	valium = 1.0;
	
	int numDudNeighbors = 0;
	//search neighbor verts in order to see if we are near an edge
	//if so, clamp to the surface closest to us
	if(depth < epsilon || (1.0 - depth) < epsilon){
		float depthDif = 1.0f;
		float nearestDepth = depth;
		for(int i = 0; i < 8; i++){
			//float depthNeighbor = depthForPoint(depthTexCoord + neighbors[i]);
			float depthNeighbor = neighborDepths[i];
			if(depthNeighbor >= epsilon && (1.0 - depthNeighbor) > epsilon){
				float thisDif = abs(nearestDepth - depthNeighbor);
				if(thisDif < depthDif){
					depthDif = thisDif;
					nearestDepth = depthNeighbor;
				}
			}
			else {
				numDudNeighbors++;
			}
		}

		depth = nearestDepth;
		
		if (depth < epsilon || (1.0-depth) < epsilon) {
			valium = 0.0;
		}
		if(numDudNeighbors > 6) {
			valium = 0.0;
		}
	}

		float2 imageCoordinates = Crop.xy + (basetex * Crop.zw);
	
		float z = depth * (FarClip - NearClip) + NearClip;
	
	float4 vertOut = float4(  ( imageCoordinates.x * ImageDimensions.x - PrincipalPoint.x )  * z / depthFocalLength.x,
	(imageCoordinates.y * ImageDimensions.y - PrincipalPoint.y ) * z /depthFocalLength.y , z , vertIn.w );
	
	vertOut = mul(extrinsics,vertOut);
	//transform position
    // Out.Pos = mul(PosO, tWVP);
    Out.Pos = mul(vertOut, tWVP);
	//Out.Pos  = vertOut;
    //transform texturecoordinates
    Out.ColTexCd = mul(colorTexCoord, tTex);
	//float2 depthTexCoord = DepthTexCd;
	//float2 colorTexCoord = ColorTexCd;
	Out.DepTexCd = depthTexCoord;
	Out.valid = valium;
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float4 col = tex2D(Samp, In.ColTexCd);
	float3 depth = tex2D ( Samp, In.DepTexCd).rgb;
	float3 depthhsv = rgb2hsv(depth);
	col.a = depthhsv.g * depthhsv.b ;
	In.valid *= (depthhsv.r > epsilon && depthhsv.g > filters.x && depthhsv.b > filters.y) ? 1.0 : 0.0;
	//float4 ff = depthForPoint(In.DepTexCd);
    return col+0.3
	
	;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TSimpleShader
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
